package com.edesarrollo.fluttermercadopago.flutter_mercadopago;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static android.app.Activity.RESULT_CANCELED;

import com.mercadopago.android.px.core.MercadoPagoCheckout;
import com.mercadopago.android.px.internal.util.ViewUtils;
import com.mercadopago.android.px.model.Payment;
import com.mercadopago.android.px.model.exceptions.MercadoPagoError;

/** FlutterMercadopagoPlugin */
public class FlutterMercadopagoPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  private static Activity activity = null;
  private static final int REQ_CODE_CHECKOUT = 1;
  private static Result result = null;

  FlutterMercadopagoPlugin(Activity activity) {
    this.activity = activity;
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "flutter_mercadopago");
    channel.setMethodCallHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_mercadopago");
    channel.setMethodCallHandler(new FlutterMercadopagoPlugin(registrar.activity()));

    registrar.addActivityResultListener(new PluginRegistry.ActivityResultListener() {
      @Override
      public boolean onActivityResult(final int requestCode, final int resultCode, final Intent data) {
        resolveCheckoutResult(activity, requestCode, resultCode, data, REQ_CODE_CHECKOUT);
        return true;
      }
    });
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    this.result = result;
    switch(call.method) {
      case "getPlatformVersion":
        result.success("Android: " + android.os.Build.VERSION.RELEASE);
      break;
      case "MercadoPagoStartForPayment":
        String publicKey = (String) call.argument("publicKey");
        String preferenceID = (String) call.argument("preferenceId");

        final MercadoPagoCheckout checkout = new MercadoPagoCheckout
                .Builder(publicKey, preferenceID)
                .build();

        checkout.startPayment(activity, REQ_CODE_CHECKOUT);

        break;
      default:
        result.notImplemented();
    }
  }
  
  public static void resolveCheckoutResult(final Activity context, final int requestCode, final int resultCode,
                                           final Intent data, final int reqCodeCheckout) {
    ViewUtils.showRegularLayout(context);

    if (requestCode == reqCodeCheckout) {
      if (resultCode == MercadoPagoCheckout.PAYMENT_RESULT_CODE) {
        final Payment payment = (Payment) data.getSerializableExtra(MercadoPagoCheckout.EXTRA_PAYMENT_RESULT);
        if(result != null) {
          result.success(payment.getId() + "");
        }
      }
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
